color_catalog_files = [
    "css_colors.yaml",
    "crayola_bright.yaml",
    "crayola_colors.yaml",
    "crayola_fluorescent.yaml",
    "crayola_fragrance.yaml",
    "crayola_jewelry.yaml",
    "crayola_magic.yaml",
    "crayola_metallic.yaml",
    "crayola_scary.yaml",
    "crayola_silver.yaml",
]


__all__ = ["color_catalog_files"]
